/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.impl;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.util.Misc;


/**
 * @author Guillaume Porcher
 *
 * Chainer for Queriers :
 *  - for methods that returns only one object, search for the first matching object
 *  - for methods that returns a collection: search for all objects in all return queriers.
 */
public class QuerierChainer implements Querier {

  private List<Querier> queriers;

  public QuerierChainer(List<Querier> queriers) {
    this.queriers = queriers;
  }

  public ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID) {
    for (Querier querier : queriers) {
      ProcessFullInstance processInst = querier.getProcessInstance(instanceUUID);
      if (processInst != null) {
        return processInst;
      }
    }
    return null;
  }

  public Set<ProcessFullInstance> getProcessInstances() {
    Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (Querier querier : queriers) {
      Set<ProcessFullInstance> tmp = querier.getProcessInstances();
      if (tmp != null && !tmp.isEmpty()) {
        processInsts.addAll(tmp);
      }
    }
    return processInsts;
  }
  
  public Set<ProcessFullInstance> getProcessInstances(
			Collection<ProcessInstanceUUID> instanceUUIDs) {
	  Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
	    for (Querier querier : queriers) {
	      Set<ProcessFullInstance> tmp = querier.getProcessInstances(instanceUUIDs);
	      if (tmp != null && !tmp.isEmpty()) {
	        processInsts.addAll(tmp);
	      }
	    }
	    return processInsts;
	}
  
  public Set<ProcessFullInstance> getProcessInstancesWithTaskState(Collection<ActivityState> acitivtyStates) {
    Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (Querier querier : queriers) {
      Set<ProcessFullInstance> tmp = querier.getProcessInstancesWithTaskState(acitivtyStates);
      if (tmp != null && !tmp.isEmpty()) {
        processInsts.addAll(tmp);
      }
    }
    return processInsts;
  }
  
  
  public Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processUUI) {
    Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    for (Querier querier : queriers) {
      processInsts.addAll(querier.getProcessInstances(processUUI));
    }
    return processInsts;
  }

  public ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId, String activityInstanceId) {
    for (Querier querier : queriers) {
      ActivityFullInstance<ActivityBody> activityInst = 
        querier.getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
      if (activityInst != null) {
        return activityInst;
      }
    }
    return null;
  }

  public Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) {
    Set<ActivityFullInstance<ActivityBody>> activityInsts = new HashSet<ActivityFullInstance<ActivityBody>>();
    for (Querier querier : queriers) {
      activityInsts.addAll(querier.getActivityInstances(instanceUUID));
    }
    return activityInsts;
  }

  public ActivityFullInstance< TaskInstance > getTaskInstance(TaskUUID taskUUID) {
    for (Querier querier : queriers) {
      ActivityFullInstance< TaskInstance > activity = querier.getTaskInstance(taskUUID);
      if (activity != null) {
        return activity;
      }
    }
    return null;
  }

  public Set<ProcessFullDefinition> getProcesses() {
    Set<ProcessFullDefinition> processes = new HashSet<ProcessFullDefinition>();
    for (Querier querier : queriers) {
      Set<ProcessFullDefinition> querierProcesses = querier.getProcesses();
      if (querierProcesses != null && !querierProcesses.isEmpty()) {
        processes.addAll(querierProcesses);
      }
    }
    return processes;
  }

  public Set<ProcessFullDefinition> getProcesses(String processId) {
    Set<ProcessFullDefinition> processes = new HashSet<ProcessFullDefinition>();
    for (Querier querier : queriers) {
      Set<ProcessFullDefinition> querierProcesses = querier.getProcesses(processId);
      if (querierProcesses != null && !querierProcesses.isEmpty()) {
        processes.addAll(querierProcesses);
      }
    }
    return processes;
  }

  public Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState) {
    Set<ProcessFullDefinition> processes = new HashSet<ProcessFullDefinition>();
    for (Querier querier : queriers) {
      Set<ProcessFullDefinition> querierProcesses = querier.getProcesses(processState);
      if (querierProcesses != null && !querierProcesses.isEmpty()) {
        processes.addAll(querierProcesses);
      }
    }
    return processes;
  }

  public Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) {
    Set<ProcessFullDefinition> processes = new HashSet<ProcessFullDefinition>();
    for (Querier querier : queriers) {
      Set<ProcessFullDefinition> querierProcesses = querier.getProcesses(processId, processState);
      if (querierProcesses != null && !querierProcesses.isEmpty()) {
        processes.addAll(querierProcesses);
      }
    }
    return processes;
  }

  public ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID) {
    for (Querier querier : queriers) {
      ProcessFullDefinition process = querier.getProcess(processUUID);
      if (process != null) {
        return process;
      }
    }
    return null;
  }
  
  public ProcessFullDefinition getProcess(String processId, String version) {
    for (Querier querier : queriers) {
      ProcessFullDefinition process = querier.getProcess(processId, version);
      if (process != null) {
        return process;
      }
    }
    return null;
  }

  public ProcessFullDefinition getLastDeployedProcess(String processId, ProcessDefinition.ProcessState processState) {
	  Misc.checkArgsNotNull(processId, processState);
    Set<ProcessFullDefinition> processes = getProcesses(processId, processState);
    ProcessFullDefinition lastProcess = null;
    for (ProcessFullDefinition process : processes) {
      if (lastProcess == null) {
        lastProcess = process;
      } else if (process.getDeployedDate().after(lastProcess.getDeployedDate())) {
        lastProcess = process;
      }
    }
    return lastProcess;
  }

  public Set<ProcessFullDefinition> getDependentProcesses(ProcessDefinitionUUID processUUID) {
    Set<ProcessFullDefinition> dependentProcesses = new HashSet<ProcessFullDefinition>();
	    for (Querier querier : queriers) {
	      Set<ProcessFullDefinition> querierDependentProcesses = querier.getDependentProcesses(processUUID);
	      if (querierDependentProcesses != null && !querierDependentProcesses.isEmpty()) {
	        dependentProcesses.addAll(querierDependentProcesses);
	      }
	    }
    return dependentProcesses;
  }

  public ActivityFullInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityInstanceUUID) {
    for (Querier querier : queriers) {
      ActivityFullInstance<ActivityBody> activity = querier.getActivityInstance(activityInstanceUUID);
      if (activity != null) {
        return activity;
      }
    }
    return null;
  }

  public Set<ActivityFullInstance<TaskInstance>> getTaskInstances(
      ProcessInstanceUUID instanceUUID) {
    Set<ActivityFullInstance< TaskInstance >> activities = new HashSet<ActivityFullInstance< TaskInstance >>();
    for (Querier querier : queriers) {
      activities.addAll(querier.getTaskInstances(instanceUUID));
    }
    return activities;
  }
  
  public Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(
      String userId, ProcessInstanceUUID instanceUUID, ActivityState taskState) {
    Set<ActivityFullInstance< TaskInstance >> activities = new HashSet<ActivityFullInstance< TaskInstance >>();
    for (Querier querier : queriers) {
      activities.addAll(querier.getUserInstanceTasks(userId, instanceUUID, taskState));
    }
    return activities;
  }
  
  public Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId,
      ActivityState taskState) {
    Set<ActivityFullInstance< TaskInstance >> activities = new HashSet<ActivityFullInstance< TaskInstance >>();
    for (Querier querier : queriers) {
      activities.addAll(querier.getUserTasks(userId, taskState));
    }
    return activities;
  }

  public ActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID) {
    for (Querier querier : queriers) {
      ActivityDefinition activity = querier.getActivity(activityDefinitionUUID);
      if (activity != null) {
        return activity;
      }
    }
    return null;
  }


}
